import React, { useState } from 'react'
import { useTranslation } from 'react-i18next'
import { Platform, StyleProp, StyleSheet, View, ViewStyle } from 'react-native'
import { Calendar, LocaleConfig } from 'react-native-calendars'

import { getLocale } from '@holi/core/i18n/helpers/dateHelper'
import { HoliIcon } from '@holi/icons/src/HoliIcon'
import { ArrowLeftS, ArrowRightS } from '@holi/icons/src/generated'
import HoliText from '@holi/ui/components/atoms/HoliText'
import HoliButton from '@holi/ui/components/molecules/HoliButton'
import HoliMandatoryFieldStar from '@holi/ui/components/molecules/HoliMandatoryFieldStar'
import { getDateObjFromYYYMMDD, localizeDate } from '@holi/ui/helper/date'
import { colors } from '@holi/ui/styles/globalVars'

LocaleConfig.locales['de'] = {
  monthNames: [
    'Januar',
    'Februar',
    'März',
    'April',
    'Mai',
    'Juni',
    'Juli',
    'August',
    'September',
    'Oktober',
    'November',
    'Dezember',
  ],
  monthNamesShort: ['Jan.', 'Feb.', 'März', 'Apr.', 'Mai', 'Juni', 'Juli', 'Aug.', 'Sept.', 'Okt.', 'Nov.', 'Dez.'],
  dayNames: ['Sonntag', 'Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag'],
  dayNamesShort: ['So.', 'Mo.', 'Di.', 'Mi.', 'Do.', 'Fr.', 'Sa.'],
  today: 'Heute',
}
export interface HoliDatePickerProps {
  onChange: (date: string) => void
  title: string
  label?: string
  value?: string
  description?: string
  placeholder?: string
  style?: StyleProp<ViewStyle>
  testID?: string
  isRequired?: boolean
  showOptionIcon?: boolean
}

const HoliDatePicker = ({
  onChange = () => null,
  title,
  label,
  value,
  placeholder,
  style,
  testID = 'holi-date-picker',
  isRequired,
}: HoliDatePickerProps) => {
  const [hovered, setHovered] = useState(false)
  const [visible, setVisible] = useState(false)

  const { i18n } = useTranslation()
  const currentLanguage = getLocale(i18n.language || '')

  const displayValue = value && localizeDate(getDateObjFromYYYMMDD(value), currentLanguage)

  const hoverPropsForWeb = Platform.select({
    web: {
      onMouseEnter: () => setHovered(true),
      onMouseOut: () => setHovered(false),
    },
  })

  return (
    <>
      <View style={style}>
        {!!label && (
          <View style={styles.labelWrapper}>
            <HoliText size="small" bold style={styles.label}>
              {label}
              <HoliMandatoryFieldStar show={!!isRequired} />
            </HoliText>
          </View>
        )}
        <HoliButton
          label={title}
          testID={`btn-${testID}`}
          transparent
          contentStyle={[styles.border, styles.button, hovered && styles.buttonHovered]}
          onPress={() => setVisible((visible) => !visible)}
          {...hoverPropsForWeb}
        >
          {value ? (
            <View style={styles.selectedDateContainer}>
              <HoliText ellipsizeMode="tail" numberOfLines={1} style={styles.selectedDateTitle}>
                {displayValue}
              </HoliText>
            </View>
          ) : (
            <HoliText style={styles.placeholder}>{placeholder}</HoliText>
          )}
        </HoliButton>
      </View>
      {visible && (
        <Calendar
          style={[styles.border, styles.calendar]}
          renderArrow={(direction) => <HoliIcon icon={direction === 'left' ? ArrowLeftS : ArrowRightS} />}
          onDayPress={(day) => {
            onChange(day.dateString)
            setVisible(false)
          }}
          markedDates={
            value
              ? {
                  [value]: { selected: true, disableTouchEvent: true, selectedColor: colors.brandColorB30 },
                }
              : {}
          }
        />
      )}
    </>
  )
}

export default HoliDatePicker

const styles = StyleSheet.create({
  labelWrapper: {
    marginBottom: 6,
    flexDirection: 'row',
  },
  label: {
    color: colors.foreground30,
  },
  border: {
    borderRadius: 4,
    borderWidth: 2,
    borderColor: colors.background30,
  },
  button: {
    paddingVertical: 16,
    paddingHorizontal: 12,
    flexDirection: 'row',
    justifyContent: 'space-between',
    alignItems: 'center',
    backgroundColor: colors.background10,
    color: colors.foreground30,
  },
  calendar: {
    marginTop: 4,
  },
  buttonHovered: {
    borderColor: colors.foreground20,
  },
  selectedDateContainer: {
    flexDirection: 'row',
    alignItems: 'center',
  },
  selectedDateTitle: {
    marginLeft: 12,
  },
  placeholder: {
    color: colors.foreground20,
  },
})
